@@ -14,14 +14,14 @@
 
 import os
 import sys
-from cStringIO import StringIO
+import cStringIO
 
 
 def _get_cli_output():
     stdout_org = sys.stdout
-    sys.stdout = output = StringIO()
-    from manilaclient.shell import OpenStackManilaShell
-    shell = OpenStackManilaShell()
+    sys.stdout = output = cStringIO.StringIO()
+    from manilaclient import shell
+    shell = shell.OpenStackManilaShell()
     shell.main(None)
     sys.stdout = stdout_org
     output.seek(0)
@@ -30,7 +30,7 @@ def _get_cli_output():
 
 def builder_inited(app):
     # generate the missing rst files
-    with open(os.path.join(app.env.srcdir, "manila_cli_output.rst.inc"), "w") as f:
+    with open(os.path.join(app.env.srcdir, "cli/manila_cli_output.rst.inc"), "w") as f:
         f.write("``manila help``::\n\n")
         f.write("\n".join(_get_cli_output()))
         f.write("\n")
