@@ -19,15 +19,15 @@ import re
 import warnings
 
 import manilaclient
+from manilaclient.common._i18n import _
+from manilaclient.common import cliutils
 from manilaclient.common import constants
 from manilaclient import exceptions
-from manilaclient.openstack.common._i18n import _
-from manilaclient.openstack.common import cliutils
 from manilaclient import utils
 
 LOG = logging.getLogger(__name__)
 
-MAX_VERSION = '2.22'
+MAX_VERSION = '2.46'
 MIN_VERSION = '2.0'
 DEPRECATED_VERSION = '1.0'
 _VERSIONED_METHOD_MAP = {}
