@@ -22,9 +22,10 @@ Base utilities to build API operation managers and objects on top of.
 import contextlib
 import hashlib
 import os
+from six.moves.urllib import parse
 
+from manilaclient.common import cliutils
 from manilaclient import exceptions
-from manilaclient.openstack.common import cliutils
 from manilaclient import utils
 
 
@@ -73,8 +74,12 @@ class Manager(utils.HookableMixin):
 
         with self.completion_cache('human_id', obj_class, mode="w"):
             with self.completion_cache('uuid', obj_class, mode="w"):
-                return [obj_class(self, res, loaded=True)
-                        for res in data if res]
+                resource = [obj_class(self, res, loaded=True)
+                            for res in data if res]
+                if 'count' in body:
+                    return resource, body['count']
+                else:
+                    return resource
 
     @contextlib.contextmanager
     def completion_cache(self, cache_type, obj_class, mode):
@@ -174,6 +179,17 @@ class Manager(utils.HookableMixin):
             else:
                 return self.resource_class(self, body)
 
+    def _build_query_string(self, search_opts):
+        query_string = ""
+        if search_opts:
+            search_opts = utils.unicode_key_value_to_string(search_opts)
+            params = sorted(
+                [(k, v) for (k, v) in search_opts.items() if v])
+            if params:
+                query_string = "?%s" % parse.urlencode(params)
+
+        return query_string
+
 
 class ManagerWithFind(Manager):
     """Like a `Manager`, but with additional `find()`/`findall()` methods."""
@@ -202,7 +218,8 @@ class ManagerWithFind(Manager):
         found = []
         searches = list(kwargs.items())
 
-        for obj in self.list():
+        search_opts = {'all_tenants': 1}
+        for obj in self.list(search_opts=search_opts):
             try:
                 if all(getattr(obj, attr) == value
                        for (attr, value) in searches):
@@ -212,5 +229,5 @@ class ManagerWithFind(Manager):
 
         return found
 
-    def list(self):
+    def list(self, search_opts=None):
         raise NotImplementedError
