@@ -26,34 +26,57 @@ from manilaclient import api_versions
 # "auth_opts" are used by functional tests that are located in
 # directory "%project_root%/manilaclient/tests/functional"
 auth_opts = [
+    # Options for user with 'member' role.
     cfg.StrOpt("username",
-               default=None,
                help="This should be the username of a user WITHOUT "
                     "administrative privileges."),
     cfg.StrOpt("tenant_name",
-               default=None,
                help="The non-administrative user's tenant name."),
     cfg.StrOpt("password",
-               default=None,
+               secret=True,
                help="The non-administrative user's password."),
     cfg.StrOpt("auth_url",
-               default=None,
                help="URL for where to find the OpenStack Identity public "
                     "API endpoint."),
+    cfg.StrOpt("project_domain_name",
+               help=("Project domain Name of user with 'member' role "
+                     "as specified for auth v3.")),
+    cfg.StrOpt("project_domain_id",
+               help=("Project domain ID of user with 'member' role "
+                     "as specified for auth v3.")),
+    cfg.StrOpt("user_domain_name",
+               help=("User domain Name of user with 'member' role "
+                     "as specified for auth v3.")),
+    cfg.StrOpt("user_domain_id",
+               help=("User domain ID of user with 'member' role "
+                     "as specified for auth v3.")),
+
+    # Options for user with 'admin' role.
     cfg.StrOpt("admin_username",
-               default=None,
                help="This should be the username of a user WITH "
                     "administrative privileges."),
     cfg.StrOpt("admin_tenant_name",
-               default=None,
                help="The administrative user's tenant name."),
     cfg.StrOpt("admin_password",
-               default=None,
+               secret=True,
                help="The administrative user's password."),
     cfg.StrOpt("admin_auth_url",
-               default=None,
                help="URL for where to find the OpenStack Identity admin "
                     "API endpoint."),
+    cfg.StrOpt("admin_project_domain_name",
+               help=("Project domain Name of user with 'admin' role "
+                     "as specified for auth v3.")),
+    cfg.StrOpt("admin_project_domain_id",
+               help=("Project domain ID of user with 'admin' role "
+                     "as specified for auth v3.")),
+    cfg.StrOpt("admin_user_domain_name",
+               help=("User domain Name of user with 'admin' role "
+                     "as specified for auth v3.")),
+    cfg.StrOpt("admin_user_domain_id",
+               help=("User domain ID of user with 'admin' role "
+                     "as specified for auth v3.")),
+
+    # Other auth options
     cfg.BoolOpt("insecure",
                 default=False,
                 help="Disable SSL certificate verification."),
@@ -84,16 +107,13 @@ share_opts = [
                     "Manilaclient functional tests. Defaults to "
                     "manilaclient's max supported API microversion."),
     cfg.StrOpt("share_network",
-               default=None,
                help="Share network Name or ID, that will be used for shares. "
                     "Some backend drivers require a share network for share "
                     "creation."),
     cfg.StrOpt("admin_share_network",
-               default=None,
                help="Share network Name or ID, that will be used for shares "
                     "in admin tenant."),
     cfg.StrOpt("share_type",
-               default=None,
                help="Share type Name or ID, that will be used with share "
                     "creation scheduling. Optional."),
     cfg.ListOpt("enable_protocols",
@@ -122,7 +142,7 @@ share_opts = [
                      "Allowed share protocols: nfs, cifs, cephfs, glusterfs, "
                      "hdfs."),
     cfg.StrOpt("username_for_user_rules",
-               default="TESTDOMAIN\\Administrator",
+               default="stack",
                help="Username, that will be used in share access tests for "
                     "user type of access."),
     cfg.BoolOpt("run_snapshot_tests",
@@ -130,6 +150,23 @@ share_opts = [
                 help="Defines whether to run tests that use share snapshots "
                      "or not. Disable this feature if used driver doesn't "
                      "support it."),
+    cfg.BoolOpt("run_share_servers_tests",
+                default=True,
+                help="Defines whether to run tests that use share servers "
+                     "or not. Disable this feature if used driver doesn't "
+                     "support it or when autodeletion of share servers "
+                     "is enabled."),
+    cfg.BoolOpt("run_migration_tests",
+                default=False,
+                help="Defines whether to run share migration tests or not. "
+                     "Disable this feature if there is no more than one "
+                     "storage pool being tested or if used driver does not "
+                     "support it."),
+    cfg.BoolOpt("run_mount_snapshot_tests",
+                default=False,
+                help="Defines whether to run mountable snapshots tests or "
+                     "not. Disable this feature if used driver doesn't "
+                     "support it."),
 ]
 
 # 2. Generate config
