@@ -34,14 +34,15 @@ import six
 
 from manilaclient import api_versions
 from manilaclient import client
+from manilaclient.common import cliutils
 from manilaclient.common import constants
 from manilaclient import exceptions as exc
 import manilaclient.extension
-from manilaclient.openstack.common import cliutils
 from manilaclient.v2 import shell as shell_v2
 
 DEFAULT_OS_SHARE_API_VERSION = api_versions.MAX_VERSION
 DEFAULT_MANILA_ENDPOINT_TYPE = 'publicURL'
+DEFAULT_MAJOR_OS_SHARE_API_VERSION = "2"
 V1_MAJOR_VERSION = '1'
 V2_MAJOR_VERSION = '2'
 
@@ -451,7 +452,7 @@ class OpenStackManilaShell(object):
         logging.basicConfig(level=logging.DEBUG, format=streamformat)
         logging.getLogger('requests.packages.urllib3.connectionpool'
                           ).setLevel(logging.WARNING)
-        logging.getLogger('keystoneclient.session').setLevel(logging.WARNING)
+        logging.getLogger('keystoneauth1.session').setLevel(logging.WARNING)
 
     def _build_subcommands_and_extensions(self,
                                           os_api_version,
@@ -464,6 +465,12 @@ class OpenStackManilaShell(object):
         self.parser = self.get_subcommand_parser(
             os_api_version.get_major_version())
 
+        if argv and len(argv) > 1 and '--help' in argv:
+            argv = [x for x in argv if x != '--help']
+            if argv[0] in self.subcommands:
+                self.subcommands[argv[0]].print_help()
+                return False
+
         if options.help or not argv:
             self.parser.print_help()
             return False
@@ -496,8 +503,19 @@ class OpenStackManilaShell(object):
             self.do_bash_completion(args)
             return 0
 
+        if not options.os_share_api_version:
+            api_version = api_versions.get_api_version(
+                DEFAULT_MAJOR_OS_SHARE_API_VERSION)
+        else:
+            api_version = api_versions.get_api_version(
+                options.os_share_api_version)
+
+        major_version_string = six.text_type(api_version.ver_major)
         os_service_type = args.service_type
-        os_endpoint_type = args.endpoint_type
+        if not os_service_type:
+            os_service_type = constants.SERVICE_TYPES[major_version_string]
+
+        os_endpoint_type = args.endpoint_type or DEFAULT_MANILA_ENDPOINT_TYPE
 
         client_args = dict(
             username=args.os_username,
@@ -507,9 +525,9 @@ class OpenStackManilaShell(object):
             insecure=args.insecure,
             region_name=args.os_region_name,
             tenant_id=args.os_project_id or args.os_tenant_id,
-            endpoint_type=DEFAULT_MANILA_ENDPOINT_TYPE,
+            endpoint_type=os_endpoint_type,
             extensions=self.extensions,
-            service_type=constants.V1_SERVICE_TYPE,
+            service_type=os_service_type,
             service_name=args.service_name,
             retries=options.retries,
             http_log_debug=args.debug,
