@@ -23,14 +23,18 @@ from tempest.lib.cli import output_parser
 from tempest.lib.common.utils import data_utils
 from tempest.lib import exceptions as tempest_lib_exc
 
+from manilaclient.common import constants
 from manilaclient import config
 from manilaclient.tests.functional import exceptions
 from manilaclient.tests.functional import utils
 
 CONF = config.CONF
+MESSAGE = 'message'
 SHARE = 'share'
 SHARE_TYPE = 'share_type'
 SHARE_NETWORK = 'share_network'
+SHARE_SERVER = 'share_server'
+SNAPSHOT = 'snapshot'
 
 
 def not_found_wrapper(f):
@@ -39,9 +43,10 @@ def not_found_wrapper(f):
         try:
             return f(self, *args, **kwargs)
         except tempest_lib_exc.CommandFailed as e:
-            if re.search('No (\w+) with a name or ID', e.stderr):
-                # Raise appropriate 'NotFound' error
-                raise tempest_lib_exc.NotFound()
+            for regexp in ('No (\w+) with a name or ID', 'not(.*){0,5}found'):
+                if re.search(regexp, e.stderr):
+                    # Raise appropriate 'NotFound' error
+                    raise tempest_lib_exc.NotFound()
             raise
 
     return wrapped_func
@@ -123,10 +128,14 @@ class ManilaCLIClient(base.CLIClient):
             func = self.is_share_type_deleted
         elif res_type == SHARE_NETWORK:
             func = self.is_share_network_deleted
+        elif res_type == SHARE_SERVER:
+            func = self.is_share_server_deleted
         elif res_type == SHARE:
             func = self.is_share_deleted
         elif res_type == SNAPSHOT:
             func = self.is_snapshot_deleted
+        elif res_type == MESSAGE:
+            func = self.is_message_deleted
         else:
             raise exceptions.InvalidResource(message=res_type)
 
@@ -141,37 +150,95 @@ class ManilaCLIClient(base.CLIClient):
             raise exceptions.ResourceReleaseFailed(
                 res_type=res_type, res_id=res_id)
 
+    def list_availability_zones(self, columns=None, microversion=None):
+        """List availability zones.
+
+        :param columns: comma separated string of columns.
+            Example, "--columns id,name"
+        :param microversion: API microversion that should be used.
+        """
+        cmd = 'availability-zone-list'
+        if columns is not None:
+            cmd += ' --columns ' + columns
+        azs_raw = self.manila(cmd, microversion=microversion)
+        azs = output_parser.listing(azs_raw)
+        return azs
+
     # Share types
 
     def create_share_type(self, name=None, driver_handles_share_servers=True,
-                          snapshot_support=True, is_public=True,
-                          microversion=None):
+                          snapshot_support=None,
+                          create_share_from_snapshot=None,
+                          revert_to_snapshot=None, mount_snapshot=None,
+                          is_public=True, microversion=None, extra_specs=None,
+                          description=None):
         """Creates share type.
 
         :param name: text -- name of share type to use, if not set then
             autogenerated will be used
+        :param description: text -- description of share type to use.
+            Default is None.
         :param driver_handles_share_servers: bool/str -- boolean or its
             string alias. Default is True.
         :param snapshot_support: bool/str -- boolean or its
-            string alias. Default is True.
+            string alias. Default is None.
         :param is_public: bool/str -- boolean or its string alias. Default is
             True.
+        :param extra_specs: -- dictionary of extra specs Default is None.
+        :param create_share_from_snapshot: -- boolean or its string
+            alias. Default is None.
+        :param revert_to_snapshot: -- boolean or its string alias. Default is
+            None.
+        :param mount_snapshot: -- boolean or its string alias. Default is None.
         """
         if name is None:
             name = data_utils.rand_name('manilaclient_functional_test')
         dhss = driver_handles_share_servers
         if not isinstance(dhss, six.string_types):
             dhss = six.text_type(dhss)
-        if not isinstance(snapshot_support, six.string_types):
-            snapshot_support = six.text_type(snapshot_support)
         if not isinstance(is_public, six.string_types):
             is_public = six.text_type(is_public)
-        cmd = ('type-create %(name)s %(dhss)s --is-public %(is_public)s '
-               '--snapshot-support %(snapshot_support)s') % {
-                   'name': name, 'dhss': dhss, 'is_public': is_public,
-                   'snapshot_support': snapshot_support}
+
+        cmd = ('type-create %(name)s %(dhss)s --is-public %(is_public)s ') % {
+            'name': name, 'dhss': dhss, 'is_public': is_public}
+
+        if description is not None:
+            cmd += " --description " + description
+
+        if snapshot_support is not None:
+            if not isinstance(snapshot_support, six.string_types):
+                snapshot_support = six.text_type(snapshot_support)
+            cmd += " --snapshot-support " + snapshot_support
+
+        if create_share_from_snapshot is not None:
+            if not isinstance(create_share_from_snapshot, six.string_types):
+                create_share_from_snapshot = six.text_type(
+                    create_share_from_snapshot)
+            cmd += (" --create-share-from-snapshot-support " +
+                    create_share_from_snapshot)
+
+        if revert_to_snapshot is not None:
+            if not isinstance(revert_to_snapshot, six.string_types):
+                revert_to_snapshot = six.text_type(
+                    revert_to_snapshot)
+            cmd += (" --revert-to-snapshot-support " + revert_to_snapshot)
+
+        if mount_snapshot is not None:
+            if not isinstance(mount_snapshot, six.string_types):
+                mount_snapshot = six.text_type(
+                    mount_snapshot)
+            cmd += (" --mount-snapshot-support " + mount_snapshot)
+
+        if extra_specs is not None:
+            extra_spec_str = ''
+            for k, v in extra_specs.items():
+                if not isinstance(v, six.string_types):
+                    extra_specs[k] = six.text_type(v)
+                extra_spec_str += "{}='{}' ".format(k, v)
+            cmd += " --extra_specs " + extra_spec_str
+
         share_type_raw = self.manila(cmd, microversion=microversion)
-        share_type = output_parser.details(share_type_raw)
+        share_type = utils.details(share_type_raw)
         return share_type
 
     @not_found_wrapper
@@ -180,16 +247,24 @@ class ManilaCLIClient(base.CLIClient):
         return self.manila(
             'type-delete %s' % share_type, microversion=microversion)
 
-    def list_share_types(self, list_all=True, columns=None, microversion=None):
+    def list_share_types(self, list_all=True, columns=None, search_opts=None,
+                         microversion=None):
         """List share types.
 
         :param list_all: bool -- whether to list all share types or only public
+        :param search_opts: dict search_opts for filter search.
         :param columns: comma separated string of columns.
             Example, "--columns id,name"
         """
         cmd = 'type-list'
         if list_all:
             cmd += ' --all'
+        if search_opts is not None:
+            extra_specs = search_opts.get('extra_specs')
+            if extra_specs:
+                cmd += ' --extra_specs'
+                for spec_key in extra_specs.keys():
+                    cmd += ' ' + spec_key + '=' + extra_specs[spec_key]
         if columns is not None:
             cmd += ' --columns ' + columns
         share_types_raw = self.manila(cmd, microversion=microversion)
@@ -231,13 +306,40 @@ class ManilaCLIClient(base.CLIClient):
             microversion=microversion)
 
     def get_project_id(self, name_or_id):
-        try:
-            # Temporary workaround for bug #1497162
-            project_id = self.openstack(
-                'project show -f value -c id %s' % name_or_id)
-        except Exception:
-            tenant = self.keystone("tenant-get %s" % name_or_id)
-            project_id = re.search("id(.*)\|(.*)\|", tenant).group(2).strip()
+        identity_api_version = '3'
+        flags = (
+            "--os-username %(username)s "
+            "--os-project-name %(project_name)s "
+            "--os-password %(password)s "
+            "--os-identity-api-version %(identity_api_version)s "
+        ) % {
+            "username": CONF.admin_username,
+            "project_name": CONF.admin_tenant_name,
+            "password": CONF.admin_password,
+            "identity_api_version": identity_api_version,
+        }
+
+        if identity_api_version == "3":
+            if CONF.admin_project_domain_name:
+                flags += (
+                    "--os-project-domain-name %s " %
+                    CONF.admin_project_domain_name)
+            elif CONF.admin_project_domain_id:
+                flags += (
+                    "--os-project-domain-id %s " %
+                    CONF.admin_project_domain_id)
+
+            if CONF.admin_user_domain_name:
+                flags += (
+                    "--os-user-domain-name %s " %
+                    CONF.admin_user_domain_name)
+            elif CONF.admin_user_domain_id:
+                flags += (
+                    "--os-user-domain-id %s " %
+                    CONF.admin_user_domain_id)
+
+        project_id = self.openstack(
+            'project show -f value -c id %s' % name_or_id, flags=flags)
         return project_id.strip()
 
     @not_found_wrapper
@@ -577,15 +679,27 @@ class ManilaCLIClient(base.CLIClient):
         shares = utils.listing(shares_raw)
         return shares
 
-    def list_share_instances(self, share_id=None, microversion=None):
+    def list_share_instances(self, share_id=None, filters=None,
+                             microversion=None):
         """List share instances.
 
         :param share_id: ID of a share to filter by.
+        :param filters: dict -- filters for listing of shares.
+            Example, input:
+                {'project_id': 'foo'}
+                {-'project_id': 'foo'}
+                {--'project_id': 'foo'}
+                {'project-id': 'foo'}
+            will be transformed to filter parameter "--export-location=foo"
         :param microversion: API microversion to be used for request.
         """
         cmd = 'share-instance-list '
         if share_id:
             cmd += '--share-id %s' % share_id
+        if filters and isinstance(filters, dict):
+            for k, v in filters.items():
+                cmd += '%(k)s=%(v)s ' % {
+                    'k': self._stranslate_to_cli_optional_param(k), 'v': v}
         share_instances_raw = self.manila(cmd, microversion=microversion)
         share_instances = utils.listing(share_instances_raw)
         return share_instances
@@ -635,6 +749,36 @@ class ManilaCLIClient(base.CLIClient):
                         "build_timeout": self.build_timeout})
                 raise tempest_lib_exc.TimeoutException(message)
 
+    def wait_for_migration_task_state(self, share_id, dest_host,
+                                      task_state_to_wait, microversion=None):
+        """Waits for a share to migrate to a certain host."""
+        statuses = ((task_state_to_wait,)
+                    if not isinstance(task_state_to_wait, (tuple, list, set))
+                    else task_state_to_wait)
+        share = self.get_share(share_id, microversion=microversion)
+        start = int(time.time())
+        while share['task_state'] not in statuses:
+            time.sleep(self.build_interval)
+            share = self.get_share(share_id, microversion=microversion)
+            if share['task_state'] in statuses:
+                break
+            elif share['task_state'] == constants.TASK_STATE_MIGRATION_ERROR:
+                raise exceptions.ShareMigrationException(
+                    share_id=share['id'], src=share['host'], dest=dest_host)
+            elif int(time.time()) - start >= self.build_timeout:
+                message = ('Share %(share_id)s failed to reach a status in'
+                           '%(status)s when migrating from host %(src)s to '
+                           'host %(dest)s within the required time '
+                           '%(timeout)s.' % {
+                               'src': share['host'],
+                               'dest': dest_host,
+                               'share_id': share['id'],
+                               'timeout': self.build_timeout,
+                               'status': six.text_type(statuses),
+                           })
+                raise tempest_lib_exc.TimeoutException(message)
+        return share
+
     @not_found_wrapper
     def _set_share_metadata(self, share, data, update_all=False,
                             microversion=None):
@@ -718,6 +862,42 @@ class ManilaCLIClient(base.CLIClient):
         snapshot = output_parser.details(snapshot_raw)
         return snapshot
 
+    @not_found_wrapper
+    def list_snapshot_export_locations(self, snapshot, columns=None,
+                                       microversion=None):
+        """List snapshot export locations.
+
+        :param snapshot: str -- Name or ID of a snapshot.
+        :param columns: str -- comma separated string of columns.
+            Example, "--columns uuid,path".
+        :param microversion: API microversion to be used for request.
+        """
+        cmd = "snapshot-export-location-list %s" % snapshot
+        if columns is not None:
+            cmd += " --columns " + columns
+        export_locations_raw = self.manila(cmd, microversion=microversion)
+        export_locations = utils.listing(export_locations_raw)
+        return export_locations
+
+    @not_found_wrapper
+    @forbidden_wrapper
+    def list_snapshot_instance_export_locations(self, snapshot_instance,
+                                                columns=None,
+                                                microversion=None):
+        """List snapshot instance export locations.
+
+        :param snapshot_instance: str -- Name or ID of a snapshot instance.
+        :param columns: str -- comma separated string of columns.
+            Example, "--columns uuid,path".
+        :param microversion: API microversion to be used for request.
+        """
+        cmd = "snapshot-instance-export-location-list %s" % snapshot_instance
+        if columns is not None:
+            cmd += " --columns " + columns
+        export_locations_raw = self.manila(cmd, microversion=microversion)
+        export_locations = utils.listing(export_locations_raw)
+        return export_locations
+
     @not_found_wrapper
     @forbidden_wrapper
     def delete_snapshot(self, snapshot, microversion=None):
@@ -801,40 +981,106 @@ class ManilaCLIClient(base.CLIClient):
                 raise tempest_lib_exc.TimeoutException(message)
 
     @not_found_wrapper
-    def list_access(self, share_id, columns=None, microversion=None):
+    def list_access(self, entity_id, columns=None, microversion=None,
+                    is_snapshot=False, metadata=None):
         """Returns list of access rules for a share.
 
-        :param share_id: str -- Name or ID of a share.
+        :param entity_id: str -- Name or ID of a share or snapshot.
         :param columns: comma separated string of columns.
             Example, "--columns access_type,access_to"
+        :param is_snapshot: Boolean value to determine if should list
+            access of a share or snapshot.
         """
-        cmd = 'access-list %s ' % share_id
+        if is_snapshot:
+            cmd = 'snapshot-access-list %s ' % entity_id
+        else:
+            cmd = 'access-list %s ' % entity_id
         if columns is not None:
             cmd += ' --columns ' + columns
+        if metadata:
+            metadata_cli = ''
+            for k, v in metadata.items():
+                metadata_cli += '%(k)s=%(v)s ' % {'k': k, 'v': v}
+            if metadata_cli:
+                cmd += ' --metadata %s ' % metadata_cli
         access_list_raw = self.manila(cmd, microversion=microversion)
         return output_parser.listing(access_list_raw)
 
     @not_found_wrapper
-    def get_access(self, share_id, access_id, microversion=None):
-        for access in self.list_access(share_id, microversion=microversion):
+    def get_access(self, share_id, access_id, microversion=None,
+                   is_snapshot=False):
+        for access in self.list_access(share_id, microversion=microversion,
+                                       is_snapshot=is_snapshot):
             if access['id'] == access_id:
                 return access
         raise tempest_lib_exc.NotFound()
 
     @not_found_wrapper
-    def access_allow(self, share_id, access_type, access_to, access_level,
-                     microversion=None):
+    def access_show(self, access_id, microversion=None):
+        raw_access = self.manila("access-show %s" % access_id,
+                                 microversion=microversion)
+        return output_parser.details(raw_access)
+
+    @not_found_wrapper
+    def access_set_metadata(self, access_id, metadata, microversion=None):
+        if not (isinstance(metadata, dict) and metadata):
+            msg = ('Provided invalid metadata for setting of access rule'
+                   ' metadata - %s' % metadata)
+            raise exceptions.InvalidData(message=msg)
+        cmd = "access-metadata %s set " % access_id
+        for k, v in metadata.items():
+            cmd += '%(k)s=%(v)s ' % {'k': k, 'v': v}
+        return self.manila(cmd, microversion=microversion)
+
+    @not_found_wrapper
+    def access_unset_metadata(self, access_id, keys, microversion=None):
+        if not (isinstance(keys, (list, tuple, set)) and keys):
+            raise exceptions.InvalidData(
+                message='Provided invalid keys - %s' % keys)
+        cmd = 'access-metadata %s unset ' % access_id
+        for key in keys:
+            cmd += '%s ' % key
+        return self.manila(cmd, microversion=microversion)
+
+    @not_found_wrapper
+    def snapshot_access_allow(self, snapshot_id, access_type, access_to,
+                              microversion=None):
         raw_access = self.manila(
-            'access-allow  --access-level %(level)s %(id)s %(type)s '
-            '%(access_to)s' % {
-                'level': access_level,
-                'id': share_id,
+            'snapshot-access-allow %(id)s %(type)s %(access_to)s' % {
+                'id': snapshot_id,
                 'type': access_type,
                 'access_to': access_to,
             },
             microversion=microversion)
         return output_parser.details(raw_access)
 
+    @not_found_wrapper
+    def snapshot_access_deny(self, snapshot_id, access_id, microversion=None):
+        return self.manila(
+            'snapshot-access-deny %(share_id)s %(access_id)s' % {
+                'share_id': snapshot_id,
+                'access_id': access_id,
+            },
+            microversion=microversion)
+
+    @not_found_wrapper
+    def access_allow(self, share_id, access_type, access_to, access_level,
+                     metadata=None, microversion=None):
+        cmd = ('access-allow  --access-level %(level)s %(id)s %(type)s '
+               '%(access_to)s' % {
+                   'level': access_level,
+                   'id': share_id,
+                   'type': access_type,
+                   'access_to': access_to})
+        if metadata:
+            metadata_cli = ''
+            for k, v in metadata.items():
+                metadata_cli += '%(k)s=%(v)s ' % {'k': k, 'v': v}
+            if metadata_cli:
+                cmd += ' --metadata %s ' % metadata_cli
+        raw_access = self.manila(cmd, microversion=microversion)
+        return output_parser.details(raw_access)
+
     @not_found_wrapper
     def access_deny(self, share_id, access_id, microversion=None):
         return self.manila(
@@ -845,15 +1091,17 @@ class ManilaCLIClient(base.CLIClient):
             microversion=microversion)
 
     def wait_for_access_rule_status(self, share_id, access_id, state='active',
-                                    microversion=None):
+                                    microversion=None, is_snapshot=False):
         access = self.get_access(
-            share_id, access_id, microversion=microversion)
+            share_id, access_id, microversion=microversion,
+            is_snapshot=is_snapshot)
 
         start = int(time.time())
         while access['state'] != state:
             time.sleep(self.build_interval)
             access = self.get_access(
-                share_id, access_id, microversion=microversion)
+                share_id, access_id, microversion=microversion,
+                is_snapshot=is_snapshot)
 
             if access['state'] == state:
                 return
@@ -870,10 +1118,11 @@ class ManilaCLIClient(base.CLIClient):
                 raise tempest_lib_exc.TimeoutException(message)
 
     def wait_for_access_rule_deletion(self, share_id, access_id,
-                                      microversion=None):
+                                      microversion=None, is_snapshot=False):
         try:
             access = self.get_access(
-                share_id, access_id, microversion=microversion)
+                share_id, access_id, microversion=microversion,
+                is_snapshot=is_snapshot)
         except tempest_lib_exc.NotFound:
             return
 
@@ -882,7 +1131,8 @@ class ManilaCLIClient(base.CLIClient):
             time.sleep(self.build_interval)
             try:
                 access = self.get_access(
-                    share_id, access_id, microversion=microversion)
+                    share_id, access_id, microversion=microversion,
+                    is_snapshot=is_snapshot)
             except tempest_lib_exc.NotFound:
                 return
 
@@ -893,7 +1143,8 @@ class ManilaCLIClient(base.CLIClient):
             if int(time.time()) - start >= self.build_timeout:
                 message = (
                     "Access rule %(access)s failed to reach deleted state "
-                    "within the required time (%s s)." % self.build_timeout)
+                    "within the required time (%(timeout)s s)." %
+                    {"access": access_id, "timeout": self.build_timeout})
                 raise tempest_lib_exc.TimeoutException(message)
 
     def reset_task_state(self, share_id, state, version=None):
@@ -903,6 +1154,47 @@ class ManilaCLIClient(base.CLIClient):
             'share': share_id,
         }, microversion=version)
 
+    def migration_start(self, share_id, dest_host, writable, nondisruptive,
+                        preserve_metadata, preserve_snapshots,
+                        force_host_assisted_migration, new_share_network=None,
+                        new_share_type=None):
+        cmd = ('migration-start %(share)s %(host)s '
+               '--writable %(writable)s --nondisruptive %(nondisruptive)s '
+               '--preserve-metadata %(preserve_metadata)s '
+               '--preserve-snapshots %(preserve_snapshots)s') % {
+            'share': share_id,
+            'host': dest_host,
+            'writable': writable,
+            'nondisruptive': nondisruptive,
+            'preserve_metadata': preserve_metadata,
+            'preserve_snapshots': preserve_snapshots,
+        }
+        if force_host_assisted_migration:
+            cmd += (' --force-host-assisted-migration %s'
+                    % force_host_assisted_migration)
+        if new_share_network:
+            cmd += ' --new-share-network %s' % new_share_network
+        if new_share_type:
+            cmd += ' --new-share-type %s' % new_share_type
+        return self.manila(cmd)
+
+    def migration_complete(self, share_id):
+        return self.manila('migration-complete %s' % share_id)
+
+    def migration_cancel(self, share_id):
+        return self.manila('migration-cancel %s' % share_id)
+
+    def migration_get_progress(self, share_id):
+        result = self.manila('migration-get-progress %s' % share_id)
+        return output_parser.details(result)
+
+    def pool_list(self, detail=False):
+        cmd = 'pool-list'
+        if detail:
+            cmd += ' --column name,host,backend,pool,capabilities'
+        response = self.manila(cmd)
+        return output_parser.listing(response)
+
     def create_security_service(self, type='ldap', name=None, description=None,
                                 dns_ip=None, server=None, domain=None,
                                 user=None, password=None, microversion=None):
@@ -986,6 +1278,40 @@ class ManilaCLIClient(base.CLIClient):
         export_locations = utils.listing(export_locations_raw)
         return export_locations
 
+    @not_found_wrapper
+    def get_snapshot_export_location(self, snapshot, export_location_uuid,
+                                     microversion=None):
+        """Returns an export location by snapshot and its UUID.
+
+        :param snapshot: str -- Name or ID of a snapshot.
+        :param export_location_uuid: str -- UUID of an export location.
+        :param microversion: API microversion to be used for request.
+        """
+        snapshot_raw = self.manila(
+            'snapshot-export-location-show %(snapshot)s %(el_uuid)s' % {
+                'snapshot': snapshot,
+                'el_uuid': export_location_uuid,
+            },
+            microversion=microversion)
+        snapshot = output_parser.details(snapshot_raw)
+        return snapshot
+
+    @not_found_wrapper
+    def get_snapshot_instance_export_location(
+            self, snapshot, export_location_uuid, microversion=None):
+        """Returns an export location by snapshot instance and its UUID.
+
+        :param snapshot: str -- Name or ID of a snapshot instance.
+        :param export_location_uuid: str -- UUID of an export location.
+        :param microversion: API microversion to be used for request.
+        """
+        snapshot_raw = self.manila(
+            'snapshot-instance-export-location-show %(snapshot)s %(el_uuid)s'
+            % {'snapshot': snapshot, 'el_uuid': export_location_uuid},
+            microversion=microversion)
+        snapshot = output_parser.details(snapshot_raw)
+        return snapshot
+
     @not_found_wrapper
     def get_share_export_location(self, share, export_location_uuid,
                                   microversion=None):
@@ -1042,3 +1368,131 @@ class ManilaCLIClient(base.CLIClient):
             microversion=microversion)
         share = output_parser.details(share_raw)
         return share
+
+    # Share servers
+
+    @not_found_wrapper
+    def get_share_server(self, share_server, microversion=None):
+        """Returns share server by its Name or ID."""
+        share_server_raw = self.manila(
+            'share-server-show %s' % share_server, microversion=microversion)
+        share_server = output_parser.details(share_server_raw)
+        return share_server
+
+    def list_share_servers(self, filters=None, columns=None,
+                           microversion=None):
+        """List share servers.
+
+        :param filters: dict -- filters for listing of share servers.
+            Example, input:
+                {'project_id': 'foo'}
+                {'-project_id': 'foo'}
+                {'--project_id': 'foo'}
+                {'project-id': 'foo'}
+            will be transformed to filter parameter "--project-id=foo"
+         :param columns: comma separated string of columns.
+            Example, "--columns id"
+        """
+        cmd = 'share-server-list '
+        if columns is not None:
+            cmd += ' --columns ' + columns
+        if filters and isinstance(filters, dict):
+            for k, v in filters.items():
+                cmd += '%(k)s=%(v)s ' % {
+                    'k': self._stranslate_to_cli_optional_param(k), 'v': v}
+        share_servers_raw = self.manila(cmd, microversion=microversion)
+        share_servers = utils.listing(share_servers_raw)
+        return share_servers
+
+    @not_found_wrapper
+    def delete_share_server(self, share_server, microversion=None):
+        """Deletes share server by its Name or ID."""
+        return self.manila('share-server-delete %s' % share_server,
+                           microversion=microversion)
+
+    def is_share_server_deleted(self, share_server_id, microversion=None):
+        """Says whether share server is deleted or not.
+
+        :param share_server: text -- ID of the share server
+        """
+        servers = self.list_share_servers(microversion=microversion)
+        for list_element in servers:
+            if share_server_id == list_element['Id']:
+                return False
+        return True
+
+    def wait_for_share_server_deletion(self, share_server, microversion=None):
+        """Wait for share server deletion by its Name or ID.
+
+        :param share_server: text -- Name or ID of share server
+        """
+        self.wait_for_resource_deletion(
+            SHARE_SERVER, res_id=share_server, interval=3, timeout=60,
+            microversion=microversion)
+
+    # user messages
+
+    def wait_for_message(self, resource_id):
+        """Waits until a message for a resource with given id exists"""
+        start = int(time.time())
+        message = None
+
+        while not message:
+            time.sleep(self.build_interval)
+            for msg in self.list_messages():
+                if msg['Resource ID'] == resource_id:
+                    return msg
+
+            if int(time.time()) - start >= self.build_timeout:
+                message = ('No message for resource with id %s was created in'
+                           ' the required time (%s s).' %
+                           (resource_id, self.build_timeout))
+                raise tempest_lib_exc.TimeoutException(message)
+
+    def list_messages(self, columns=None, microversion=None):
+        """List messages.
+
+        :param columns: str -- comma separated string of columns.
+            Example, "--columns id,resource_id".
+        :param microversion: API microversion to be used for request.
+        """
+        cmd = "message-list"
+        if columns is not None:
+            cmd += " --columns " + columns
+        messages_raw = self.manila(cmd, microversion=microversion)
+        messages = utils.listing(messages_raw)
+        return messages
+
+    @not_found_wrapper
+    def get_message(self, message, microversion=None):
+        """Returns share server by its Name or ID."""
+        message_raw = self.manila(
+            'message-show %s' % message, microversion=microversion)
+        message = output_parser.details(message_raw)
+        return message
+
+    @not_found_wrapper
+    def delete_message(self, message, microversion=None):
+        """Deletes message by its ID."""
+        return self.manila('message-delete %s' % message,
+                           microversion=microversion)
+
+    def is_message_deleted(self, message, microversion=None):
+        """Indicates whether message is deleted or not.
+
+        :param message: str -- ID of message
+        """
+        try:
+            self.get_message(message, microversion=microversion)
+            return False
+        except tempest_lib_exc.NotFound:
+            return True
+
+    def wait_for_message_deletion(self, message, microversion=None):
+        """Wait for message deletion by its ID.
+
+        :param message: text -- ID of message
+        """
+        self.wait_for_resource_deletion(
+            MESSAGE, res_id=message, interval=3, timeout=60,
+            microversion=microversion)
