@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # Copyright 2015 Mirantis Inc.
 # All Rights Reserved.
 #
@@ -43,6 +44,18 @@ class SharesListReadOnlyTest(base.BaseTestCase):
     def test_shares_list_filter_by_name(self, role):
         self.clients[role].manila('list', params='--name name')
 
+    @ddt.data('admin', 'user')
+    def test_shares_list_filter_by_export_location(self, role):
+        self.clients[role].manila('list', params='--export_location fake')
+
+    @ddt.data('admin', 'user')
+    def test_shares_list_filter_by_inexact_name(self, role):
+        self.clients[role].manila('list', params='--name~ na')
+
+    @ddt.data('admin', 'user')
+    def test_shares_list_filter_by_inexact_description(self, role):
+        self.clients[role].manila('list', params='--description~ des')
+
     @ddt.data('admin', 'user')
     def test_shares_list_filter_by_status(self, role):
         self.clients[role].manila('list', params='--status status')
@@ -61,9 +74,8 @@ class SharesListReadOnlyTest(base.BaseTestCase):
     def test_shares_list_filter_by_project_id(self, role):
         self.clients[role].manila('list', params='--project-id fake')
 
-    @ddt.data('admin', 'user')
-    def test_shares_list_filter_by_host(self, role):
-        self.clients[role].manila('list', params='--host fake')
+    def test_shares_list_filter_by_host(self):
+        self.clients['admin'].manila('list', params='--host fake')
 
     @ddt.data('admin', 'user')
     def test_shares_list_with_limit_and_offset(self, role):
@@ -145,9 +157,7 @@ class SharesListReadWriteTest(base.BaseTestCase):
         filters = filters or dict()
         shares = self.user_client.list_shares(filters=filters)
 
-        self.assertTrue(len(shares) > 1)
-        for s_id in (self.private_share['id'], self.public_share['id']):
-            self.assertTrue(any(s_id == s['ID'] for s in shares))
+        self.assertGreater(len(shares), 0)
         if filters:
             for share in shares:
                 try:
@@ -170,7 +180,7 @@ class SharesListReadWriteTest(base.BaseTestCase):
     @ddt.data(1, 0)
     def test_list_shares_for_all_tenants(self, all_tenants):
         shares = self.admin_client.list_shares(all_tenants=all_tenants)
-        self.assertTrue(len(shares) >= 1)
+        self.assertLessEqual(1, len(shares))
 
         if all_tenants:
             self.assertTrue(all('Project ID' in s for s in shares))
@@ -190,7 +200,7 @@ class SharesListReadWriteTest(base.BaseTestCase):
     @ddt.data(True, False)
     def test_list_shares_with_public(self, public):
         shares = self.user_client.list_shares(is_public=public)
-        self.assertTrue(len(shares) > 1)
+        self.assertGreater(len(shares), 1)
         if public:
             self.assertTrue(all('Project ID' in s for s in shares))
         else:
@@ -228,10 +238,6 @@ class SharesListReadWriteTest(base.BaseTestCase):
             CONF.share_network)['id']
         self._list_shares({'share_network': share_network_id})
 
-    def test_list_shares_by_host(self):
-        get = self.user_client.get_share(self.private_share['id'])
-        self._list_shares({'host': get['host']})
-
     @ddt.data(
         {'limit': 1},
         {'limit': 2},
@@ -247,3 +253,84 @@ class SharesListReadWriteTest(base.BaseTestCase):
         self.assertTrue(any(s['Name'] is not None for s in shares))
         self.assertTrue(any(s['Size'] is not None for s in shares))
         self.assertTrue(all('Description' not in s for s in shares))
+
+    @ddt.data('ID', 'Path')
+    def test_list_shares_by_export_location(self, option):
+        export_locations = self.admin_client.list_share_export_locations(
+            self.public_share['id'])
+        shares = self.admin_client.list_shares(
+            filters={'export_location': export_locations[0][option]})
+
+        self.assertEqual(1, len(shares))
+        self.assertTrue(
+            any(self.public_share['id'] == s['ID'] for s in shares))
+        for share in shares:
+            get = self.admin_client.get_share(share['ID'])
+            self.assertEqual(self.public_name, get['name'])
+
+    @ddt.data('ID', 'Path')
+    def test_list_share_instances_by_export_location(self, option):
+        export_locations = self.admin_client.list_share_export_locations(
+            self.public_share['id'])
+        share_instances = self.admin_client.list_share_instances(
+            filters={'export_location': export_locations[0][option]})
+
+        self.assertEqual(1, len(share_instances))
+
+        share_instance_id = share_instances[0]['ID']
+        except_export_locations = (
+            self.admin_client.list_share_instance_export_locations(
+                share_instance_id))
+        self.assertGreater(len(except_export_locations), 0)
+        self.assertTrue(
+            any(export_locations[0][option] == e[option] for e in
+                except_export_locations))
+
+    def test_list_share_by_export_location_with_invalid_version(self):
+        self.assertRaises(
+            exceptions.CommandFailed,
+            self.admin_client.list_shares,
+            filters={'export_location': 'fake'},
+            microversion='2.34')
+
+    def test_list_share_instance_by_export_location_invalid_version(self):
+        self.assertRaises(
+            exceptions.CommandFailed,
+            self.admin_client.list_share_instances,
+            filters={'export_location': 'fake'},
+            microversion='2.34')
+
+    @ddt.data('name', 'description')
+    def test_list_shares_by_inexact_option(self, option):
+        shares = self.user_client.list_shares(
+            filters={option + '~': option})
+
+        # We know we have to have atleast three shares.
+        # Due to test concurrency, there can be
+        # more than three shares (some created by other tests).
+        self.assertGreaterEqual(len(shares), 3)
+        self.assertTrue(
+            any(self.private_share['id'] == s['ID'] for s in shares))
+
+    def test_list_shares_by_inexact_unicode_option(self):
+        self.create_share(
+            name=u'共享名称',
+            description=u'共享描述',
+            public=True,
+            client=self.get_user_client(),
+            cleanup_in_class=True)
+        filters = {'name~': u'名称'}
+        shares = self.user_client.list_shares(filters=filters)
+        self.assertGreater(len(shares), 0)
+
+        filters = {'description~': u'描述'}
+        shares = self.user_client.list_shares(filters=filters)
+        self.assertGreater(len(shares), 0)
+
+    def test_list_shares_by_description(self):
+        shares = self.user_client.list_shares(
+            filters={'description': self.private_description})
+
+        self.assertEqual(1, len(shares))
+        self.assertTrue(
+            any(self.private_share['id'] == s['ID'] for s in shares))
