@@ -14,11 +14,12 @@
 
 import ddt
 import mock
+import six
 
 import manilaclient
 from manilaclient import api_versions
+from manilaclient.common import cliutils
 from manilaclient import exceptions
-from manilaclient.openstack.common import cliutils
 from manilaclient.tests.unit import utils
 
 
@@ -40,6 +41,7 @@ class APIVersionTestCase(utils.TestCase):
     def test_null_version(self):
         v = api_versions.APIVersion()
         self.assertTrue(v.is_null())
+        self.assertEqual(repr(v), "<APIVersion: null>")
 
     @ddt.data(
         "2",
@@ -68,15 +70,17 @@ class APIVersionTestCase(utils.TestCase):
         v5 = api_versions.APIVersion("1.0")
         v_null = api_versions.APIVersion()
 
-        self.assertTrue(v1 < v2)
-        self.assertTrue(v3 > v2)
-        self.assertTrue(v1 != v2)
-        self.assertTrue(v1 == v4)
-        self.assertTrue(v1 != v_null)
-        self.assertTrue(v5 < v1)
-        self.assertTrue(v5 < v2)
-        self.assertTrue(v_null == v_null)
+        self.assertLess(v1, v2)
+        self.assertGreater(v3, v2)
+        self.assertNotEqual(v1, v2)
+        self.assertEqual(v1, v4)
+        self.assertNotEqual(v1, v_null)
+        self.assertLess(v5, v1)
+        self.assertLess(v5, v2)
+        self.assertEqual(v_null, v_null)
         self.assertRaises(TypeError, v1.__le__, "2.1")
+        self.assertRaises(TypeError, v1.__eq__, "2.1")
+        self.assertRaises(TypeError, v1.__gt__, "2.1")
 
     def test_version_matches(self):
         v1 = api_versions.APIVersion("2.0")
@@ -115,6 +119,28 @@ class APIVersionTestCase(utils.TestCase):
         self.assertRaises(ValueError,
                           api_versions.APIVersion().get_string)
 
+    @ddt.data("2.0",
+              "2.5",
+              "2.45",
+              "3.3",
+              "3.23",
+              "2.0",
+              "3.3",
+              "4.0")
+    def test_representation(self, version):
+        version_major, version_minor = version.split('.')
+        api_version = api_versions.APIVersion(version)
+        self.assertEqual(six.text_type(api_version),
+                         ("API Version Major: %s, Minor: %s" %
+                          (version_major, version_minor)))
+        self.assertEqual(repr(api_version), "<APIVersion: %s>" % version)
+
+    def test_is_latest(self):
+        v1 = api_versions.APIVersion("1.0")
+        self.assertFalse(v1.is_latest())
+        v_latest = api_versions.APIVersion(api_versions.MAX_VERSION)
+        self.assertTrue(v_latest.is_latest())
+
 
 class GetAPIVersionTestCase(utils.TestCase):
 
@@ -260,11 +286,11 @@ class DiscoverVersionTestCase(utils.TestCase):
         manilaclient.API_MAX_VERSION = api_versions.APIVersion("2.3")
         manilaclient.API_MIN_VERSION = api_versions.APIVersion("2.1")
 
-        self.assertRaisesRegexp(exceptions.UnsupportedVersion,
-                                ".*range is '2.4' to '2.7'.*",
-                                api_versions.discover_version,
-                                self.fake_client,
-                                api_versions.APIVersion("2.3"))
+        self.assertRaisesRegex(exceptions.UnsupportedVersion,
+                               ".*range is '2.4' to '2.7'.*",
+                               api_versions.discover_version,
+                               self.fake_client,
+                               api_versions.APIVersion("2.3"))
         self.assertTrue(self.fake_client.services.server_api_version.called)
 
     def test_server_is_too_old(self):
@@ -347,8 +373,8 @@ class DiscoverVersionTestCase(utils.TestCase):
         manilaclient.API_MAX_VERSION = api_versions.APIVersion("2.5")
         manilaclient.API_MIN_VERSION = api_versions.APIVersion("2.5")
 
-        self.assertRaisesRegexp(exceptions.UnsupportedVersion,
-                                ".*range is '2.0' to '2.5'.*",
-                                api_versions.discover_version,
-                                self.fake_client,
-                                api_versions.APIVersion("1.0"))
+        self.assertRaisesRegex(exceptions.UnsupportedVersion,
+                               ".*range is '2.0' to '2.5'.*",
+                               api_versions.discover_version,
+                               self.fake_client,
+                               api_versions.APIVersion("1.0"))
