@@ -19,6 +19,7 @@ import ddt
 import mock
 
 from manilaclient import api_versions
+from manilaclient.common.apiclient import exceptions as client_exceptions
 from manilaclient import exceptions
 from manilaclient import extension
 from manilaclient.tests.unit import utils
@@ -48,11 +49,11 @@ class SharesTest(utils.TestCase):
         self.share.allow(access_type, access_to, access_level)
 
         self.share.manager.allow.assert_called_once_with(
-            self.share, access_type, access_to, access_level)
+            self.share, access_type, access_to, access_level, None)
 
     # Testcases for class ShareManager
 
-    @ddt.data('nfs', 'cifs', 'cephfs', 'glusterfs', 'hdfs')
+    @ddt.data('nfs', 'cifs', 'cephfs', 'glusterfs', 'hdfs', 'maprfs')
     def test_create_share_with_protocol(self, protocol):
         expected = {
             'size': 1,
@@ -65,7 +66,6 @@ class SharesTest(utils.TestCase):
             'share_type': None,
             'is_public': False,
             'availability_zone': None,
-            'consistency_group_id': None,
         }
         cs.shares.create(protocol, 1)
         cs.assert_called('POST', '/shares', {'share': expected})
@@ -86,7 +86,6 @@ class SharesTest(utils.TestCase):
             'share_type': None,
             'is_public': False,
             'availability_zone': None,
-            'consistency_group_id': None,
         }
         cs.shares.create('nfs', 1, share_network=share_network)
         cs.assert_called('POST', '/shares', {'share': expected})
@@ -107,7 +106,6 @@ class SharesTest(utils.TestCase):
             'share_type': 'fake_st',
             'is_public': False,
             'availability_zone': None,
-            'consistency_group_id': None,
         }
         cs.shares.create('nfs', 1, share_type=share_type)
         cs.assert_called('POST', '/shares', {'share': expected})
@@ -131,7 +129,6 @@ class SharesTest(utils.TestCase):
                 'share_network_id': None,
                 'size': 1,
                 'availability_zone': availability_zone,
-                'consistency_group_id': None,
             }
         }
         cs.shares.create('nfs', 1, is_public=is_public,
@@ -239,6 +236,35 @@ class SharesTest(utils.TestCase):
             manager._action.assert_called_once_with("unmanage", share)
             self.assertEqual("fake", result)
 
+    def test_revert_to_snapshot(self):
+
+        share = 'fake_share'
+        snapshot = 'fake_snapshot'
+        version = api_versions.APIVersion("2.27")
+        mock_microversion = mock.Mock(api_version=version)
+        manager = shares.ShareManager(api=mock_microversion)
+        mock_action = self.mock_object(
+            manager, '_action', mock.Mock(return_value='fake'))
+
+        result = manager.revert_to_snapshot(share, snapshot)
+
+        self.assertEqual('fake', result)
+        mock_action.assert_called_once_with(
+            'revert', 'fake_share', info={'snapshot_id': 'fake_snapshot'})
+
+    def test_revert_to_snapshot_not_supported(self):
+
+        share = 'fake_share'
+        snapshot = 'fake_snapshot'
+        version = api_versions.APIVersion("2.26")
+        mock_microversion = mock.Mock(api_version=version)
+        manager = shares.ShareManager(api=mock_microversion)
+
+        self.assertRaises(client_exceptions.UnsupportedVersion,
+                          manager.revert_to_snapshot,
+                          share,
+                          snapshot)
+
     @ddt.data(
         ("2.6", "os-force_delete"),
         ("2.7", "force_delete"),
@@ -265,13 +291,46 @@ class SharesTest(utils.TestCase):
         search_opts = {
             'fake_str': 'fake_str_value',
             'fake_int': 1,
+            'name~': 'fake_name',
+            'description~': 'fake_description',
         }
         cs.shares.list(detailed=False, search_opts=search_opts)
         cs.assert_called(
             'GET',
-            '/shares?fake_int=1&fake_str=fake_str_value&is_public=True')
+            '/shares?description%7E=fake_description&fake_int=1&'
+            'fake_str=fake_str_value&is_public=True&name%7E=fake_name')
+
+    @ddt.data(('id', 'b4991315-eb7d-43ec-979e-5715d4399827', True),
+              ('id', 'b4991315-eb7d-43ec-979e-5715d4399827', False),
+              ('path', 'fake_path', False),
+              ('path', 'fake_path', True))
+    @ddt.unpack
+    def test_list_shares_index_with_export_location(self, filter_type,
+                                                    value, detailed):
+        search_opts = {
+            'export_location': value,
+        }
+        cs.shares.list(detailed=detailed, search_opts=search_opts)
+        if detailed:
+            cs.assert_called(
+                'GET', ('/shares/detail?export_location_' + filter_type +
+                        '=' + value + '&is_public=True'))
+        else:
+            cs.assert_called(
+                'GET', ('/shares?export_location_' + filter_type + '='
+                        + value + '&is_public=True'))
 
     def test_list_shares_detailed(self):
+        search_opts = {
+            'with_count': 'True',
+        }
+        shares, count = cs.shares.list(detailed=True, search_opts=search_opts)
+        cs.assert_called(
+            'GET', '/shares/detail?is_public=True&with_count=True')
+        self.assertEqual(2, count)
+        self.assertEqual(1, len(shares))
+
+    def test_list_shares_detailed_with_count(self):
         cs.shares.list(detailed=True)
         cs.assert_called('GET', '/shares/detail?is_public=True')
 
@@ -320,11 +379,13 @@ class SharesTest(utils.TestCase):
          'action_name': 'os-allow_access', 'microversion': '2.0'},
         {'access_to': '1' * 4, 'access_type': 'user',
          'action_name': 'os-allow_access', 'microversion': '2.0'},
-        {'access_to': '1' * 32, 'access_type': 'user',
+        {'access_to': '1' * 255, 'access_type': 'user',
          'action_name': 'os-allow_access', 'microversion': '2.0'},
-        {'access_to': 'fake\\]{.-_\'`;}[', 'access_type': 'user',
+        {'access_to': 'fake${.-_\'`}', 'access_type': 'user',
          'action_name': 'os-allow_access', 'microversion': '2.0'},
-        {'access_to': 'MYDOMAIN\\Administrator', 'access_type': 'user',
+        {'access_to': 'MYDOMAIN-Administrator', 'access_type': 'user',
+         'action_name': 'os-allow_access', 'microversion': '2.0'},
+        {'access_to': 'test group name', 'access_type': 'user',
          'action_name': 'os-allow_access', 'microversion': '2.0'},
         {'access_to': 'x', 'access_type': 'cert',
          'action_name': 'os-allow_access', 'microversion': '2.0'},
@@ -334,12 +395,30 @@ class SharesTest(utils.TestCase):
          'action_name': 'os-allow_access', 'microversion': '2.0'},
         {'access_to': '127.0.0.1', 'access_type': 'ip',
          'action_name': 'allow_access', 'microversion': '2.7'},
+        {'access_to': 'test group name', 'access_type': 'user',
+         'action_name': 'allow_access', 'microversion': '2.7'},
         {'access_to': 'alice', 'access_type': 'cephx',
          'action_name': 'allow_access', 'microversion': '2.13'},
         {'access_to': 'alice_bob', 'access_type': 'cephx',
          'action_name': 'allow_access', 'microversion': '2.13'},
         {'access_to': 'alice bob', 'access_type': 'cephx',
          'action_name': 'allow_access', 'microversion': '2.13'},
+        {'access_to': 'test group name', 'access_type': 'user',
+         'action_name': 'allow_access', 'microversion': '2.13'},
+        {'access_to': 'AD80:0000:0000:0000:ABAA:0000:00C2:0002',
+         'access_type': 'ip', 'action_name': 'allow_access',
+         'microversion': '2.38'},
+        {'access_to': 'AD80::/36',
+         'access_type': 'ip', 'action_name': 'allow_access',
+         'microversion': '2.38'},
+        {'access_to': 'AD80:ABAA::/128',
+         'access_type': 'ip', 'action_name': 'allow_access',
+         'microversion': '2.38'},
+        {'access_to': 'ad80::abaa:0:c2:2',
+         'access_type': 'ip', 'action_name': 'allow_access',
+         'microversion': '2.38'},
+        {'access_to': 'test group name', 'access_type': 'user',
+         'action_name': 'allow_access', 'microversion': '2.38'},
     )
     @ddt.unpack
     def test_allow_access_to_share(self, access_to, access_type,
@@ -374,9 +453,9 @@ class SharesTest(utils.TestCase):
          'microversion': '2.0'},
         {'access_to': '1' * 3, 'access_type': 'user',
          'microversion': '2.0'},
-        {'access_to': '1' * 33, 'access_type': 'user',
+        {'access_to': '1' * 256, 'access_type': 'user',
          'microversion': '2.0'},
-        {'access_to': 'root^', 'access_type': 'user',
+        {'access_to': 'root+=', 'access_type': 'user',
          'microversion': '2.0'},
         {'access_to': '', 'access_type': 'cert',
          'microversion': '2.0'},
@@ -390,6 +469,12 @@ class SharesTest(utils.TestCase):
          'microversion': '2.13'},
         {'access_to': u"bj\u00F6rn", 'access_type': 'cephx',
          'microversion': '2.13'},
+        {'access_to': "AD80:0000:0000:0000:ABAA:0000:00C2:0002/65",
+         'access_type': 'ip', 'microversion': '2.38'},
+        {'access_to': "AD80:0000:0000:0000:ABAA:0000:00C2:0002*32",
+         'access_type': 'ip', 'microversion': '2.38'},
+        {'access_to': "AD80:0000:0000:0000:ABAA:0000:00C2:0002",
+         'access_type': 'ip', 'microversion': '2.37'},
     )
     @ddt.unpack
     def test_allow_access_to_share_error_invalid_access(self, access_to,
@@ -524,20 +609,24 @@ class SharesTest(utils.TestCase):
     def test_migration_start(self):
         share = "fake_share"
         host = "fake_host"
-        version = api_versions.APIVersion('2.22')
+        version = api_versions.APIVersion('2.29')
         manager = shares.ShareManager(
             api=fakes.FakeClient(api_version=version))
 
         with mock.patch.object(manager, "_action",
                                mock.Mock(return_value="fake")):
-            result = manager.migration_start(share, host, True)
+            result = manager.migration_start(
+                share, host, force_host_assisted_migration=True,
+                preserve_metadata=True, writable=True, nondisruptive=True,
+                preserve_snapshots=True)
             manager._action.assert_called_once_with(
                 'migration_start', share, {
                     "host": host,
                     "force_host_assisted_migration": True,
                     "preserve_metadata": True,
                     "writable": True,
-                    "nondisruptive": False,
+                    "nondisruptive": True,
+                    "preserve_snapshots": True,
                     "new_share_network_id": None,
                     "new_share_type_id": None,
                 })
