@@ -10,32 +10,37 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import warnings
+from debtcollector import removals
 
-from keystoneclient import adapter
+from keystoneauth1 import adapter
+from keystoneauth1 import session
 from keystoneclient import client as ks_client
-from keystoneclient import discover
-from keystoneclient import session
-import six
 
 import manilaclient
 from manilaclient.common import constants
 from manilaclient.common import httpclient
 from manilaclient import exceptions
-from manilaclient.v2 import consistency_group_snapshots as cg_snapshots
-from manilaclient.v2 import consistency_groups
+from manilaclient.v2 import availability_zones
 from manilaclient.v2 import limits
+from manilaclient.v2 import messages
 from manilaclient.v2 import quota_classes
 from manilaclient.v2 import quotas
 from manilaclient.v2 import scheduler_stats
 from manilaclient.v2 import security_services
 from manilaclient.v2 import services
+from manilaclient.v2 import share_access_rules
 from manilaclient.v2 import share_export_locations
+from manilaclient.v2 import share_group_snapshots
+from manilaclient.v2 import share_group_type_access
+from manilaclient.v2 import share_group_types
+from manilaclient.v2 import share_groups
 from manilaclient.v2 import share_instance_export_locations
 from manilaclient.v2 import share_instances
 from manilaclient.v2 import share_networks
 from manilaclient.v2 import share_replicas
 from manilaclient.v2 import share_servers
+from manilaclient.v2 import share_snapshot_export_locations
+from manilaclient.v2 import share_snapshot_instance_export_locations
 from manilaclient.v2 import share_snapshot_instances
 from manilaclient.v2 import share_snapshots
 from manilaclient.v2 import share_type_access
@@ -51,10 +56,10 @@ class Client(object):
         >>> client = Client(USERNAME, PASSWORD, PROJECT_ID, AUTH_URL)
 
     Or, alternatively, you can create a client instance using the
-    keystoneclient.session API::
+    keystoneauth1.session API::
 
         >>> from keystoneclient.auth.identity import v2
-        >>> from keystoneclient import session
+        >>> from keystoneauth1 import session
         >>> from manilaclient import client
         >>> auth = v2.Password(auth_url=AUTH_URL,
                                username=USERNAME,
@@ -68,6 +73,21 @@ class Client(object):
         >>> client.shares.list()
         ...
     """
+    @removals.removed_kwarg(
+        'share_service_name', message="Please use 'service_name' instead",
+        removal_version='2.0.0')
+    @removals.removed_kwarg(
+        'proxy_tenant_id', message="This is not used anywhere",
+        removal_version='2.0.0')
+    @removals.removed_kwarg(
+        'proxy_token', message="This is not used anywhere",
+        removal_version='2.0.0')
+    @removals.removed_kwarg(
+        'os_cache', message="Please use 'use_keyring' instead",
+        removal_version='2.0.0')
+    @removals.removed_kwarg(
+        'api_key', message="Please use 'password' instead",
+        removal_version='2.0.0')
     def __init__(self, username=None, api_key=None,
                  project_id=None, auth_url=None, insecure=False, timeout=None,
                  tenant_id=None, project_name=None, region_name=None,
@@ -115,32 +135,6 @@ class Client(object):
 
         service_name = kwargs.get("share_service_name", service_name)
 
-        def check_deprecated_arguments():
-            deprecated = {
-                'share_service_name': 'service_name',
-                'proxy_tenant_id': None,
-                'proxy_token': None,
-                'os_cache': 'use_keyring',
-                'api_key': 'password',
-            }
-
-            for arg, replacement in six.iteritems(deprecated):
-                if kwargs.get(arg, None) is None:
-                    continue
-
-                replacement_msg = ""
-
-                if replacement is not None:
-                    replacement_msg = " Use %s instead." % replacement
-
-                msg = "Argument %(arg)s is deprecated.%(repl)s" % {
-                    'arg': arg,
-                    'repl': replacement_msg
-                }
-                warnings.warn(msg)
-
-        check_deprecated_arguments()
-
         if input_auth_token and not service_catalog_url:
             msg = ("For token-based authentication you should "
                    "provide 'input_auth_token' and 'service_catalog_url'.")
@@ -205,7 +199,10 @@ class Client(object):
                                             http_log_debug=http_log_debug,
                                             api_version=self.api_version)
 
+        self.availability_zones = availability_zones.AvailabilityZoneManager(
+            self)
         self.limits = limits.LimitsManager(self)
+        self.messages = messages.MessageManager(self)
         self.services = services.ServiceManager(self)
         self.security_services = security_services.SecurityServiceManager(self)
         self.share_networks = share_networks.ShareNetworkManager(self)
@@ -216,6 +213,12 @@ class Client(object):
         self.shares = shares.ShareManager(self)
         self.share_export_locations = (
             share_export_locations.ShareExportLocationManager(self))
+        self.share_groups = share_groups.ShareGroupManager(self)
+        self.share_group_snapshots = (
+            share_group_snapshots.ShareGroupSnapshotManager(self))
+        self.share_group_type_access = (
+            share_group_type_access.ShareGroupTypeAccessManager(self))
+        self.share_group_types = share_group_types.ShareGroupTypeManager(self)
         self.share_instances = share_instances.ShareInstanceManager(self)
         self.share_instance_export_locations = (
             share_instance_export_locations.ShareInstanceExportLocationManager(
@@ -223,16 +226,20 @@ class Client(object):
         self.share_snapshots = share_snapshots.ShareSnapshotManager(self)
         self.share_snapshot_instances = (
             share_snapshot_instances.ShareSnapshotInstanceManager(self))
+        self.share_snapshot_export_locations = (
+            share_snapshot_export_locations.ShareSnapshotExportLocationManager(
+                self))
+        self.share_snapshot_instance_export_locations = (
+            share_snapshot_instance_export_locations.
+            ShareSnapshotInstanceExportLocationManager(self))
 
         self.share_types = share_types.ShareTypeManager(self)
         self.share_type_access = share_type_access.ShareTypeAccessManager(self)
         self.share_servers = share_servers.ShareServerManager(self)
         self.share_replicas = share_replicas.ShareReplicaManager(self)
         self.pools = scheduler_stats.PoolManager(self)
-        self.consistency_groups = (
-            consistency_groups.ConsistencyGroupManager(self))
-        self.cg_snapshots = (
-            cg_snapshots.ConsistencyGroupSnapshotManager(self))
+        self.share_access_rules = (
+            share_access_rules.ShareAccessRuleManager(self))
 
         self._load_extensions(extensions)
 
@@ -244,6 +251,10 @@ class Client(object):
             if extension.manager_class:
                 setattr(self, extension.name, extension.manager_class(self))
 
+    @removals.remove(
+        message="authenticate() method is deprecated. Client automatically "
+        "makes authentication call in the constructor.",
+        removal_version='2.0.0')
     def authenticate(self):
         """Authenticate against the server.
 
@@ -253,9 +264,7 @@ class Client(object):
         Returns on success; raises :exc:`exceptions.Unauthorized` if the
         credentials are wrong.
         """
-        warnings.warn("authenticate() method is deprecated. "
-                      "Client automatically makes authentication call "
-                      "in the constructor.")
+        pass
 
     def _get_keystone_client(self):
         # First create a Keystone session
@@ -266,8 +275,7 @@ class Client(object):
         ks_session = session.Session(verify=verify, cert=self.cert)
 
         # Discover the supported keystone versions using the given url
-        ks_discover = discover.Discover(
-            session=ks_session, auth_url=self.auth_url)
+        ks_discover = session.discover.Discover(ks_session, self.auth_url)
 
         # Inspect the auth_url to see the supported version. If both v3 and v2
         # are supported, then use the highest version if possible.
