@@ -15,18 +15,19 @@
 """Interface for shares extension."""
 
 import collections
+import ipaddress
+from oslo_utils import uuidutils
 import re
+import six
+from six.moves.urllib import parse
 import string
-try:
-    from urllib import urlencode  # noqa
-except ImportError:
-    from urllib.parse import urlencode  # noqa
 
 from manilaclient import api_versions
 from manilaclient import base
+from manilaclient.common.apiclient import base as common_base
 from manilaclient.common import constants
 from manilaclient import exceptions
-from manilaclient.openstack.common.apiclient import base as common_base
+from manilaclient import utils
 from manilaclient.v2 import share_instances
 
 
@@ -44,14 +45,14 @@ class Share(common_base.Resource):
         self.manager.unmanage(self, **kwargs)
 
     def migration_start(self, host, force_host_assisted_migration,
-                        preserve_metadata=True, writable=True,
-                        nondisruptive=False, new_share_network_id=None,
+                        preserve_metadata, writable, nondisruptive,
+                        preserve_snapshots, new_share_network_id=None,
                         new_share_type_id=None):
         """Migrate the share to a new host."""
         self.manager.migration_start(self, host, force_host_assisted_migration,
                                      preserve_metadata, writable,
-                                     nondisruptive, new_share_network_id,
-                                     new_share_type_id)
+                                     nondisruptive, preserve_snapshots,
+                                     new_share_network_id, new_share_type_id)
 
     def migration_complete(self):
         """Complete migration of a share."""
@@ -69,17 +70,18 @@ class Share(common_base.Resource):
         """Reset the task state of a given share."""
         self.manager.reset_task_state(self, task_state)
 
-    def delete(self, consistency_group_id=None):
+    def delete(self, share_group_id=None):
         """Delete this share."""
-        self.manager.delete(self, consistency_group_id=consistency_group_id)
+        self.manager.delete(self, share_group_id=share_group_id)
 
     def force_delete(self):
         """Delete the specified share ignoring its current state."""
         self.manager.force_delete(self)
 
-    def allow(self, access_type, access, access_level):
+    def allow(self, access_type, access, access_level, metadata=None):
         """Allow access to a share."""
-        return self.manager.allow(self, access_type, access, access_level)
+        return self.manager.allow(
+            self, access_type, access, access_level, metadata)
 
     def deny(self, id):
         """Deny access from IP to a share."""
@@ -109,6 +111,10 @@ class Share(common_base.Resource):
         """List instances of the specified share."""
         self.manager.list_instances(self)
 
+    def revert_to_snapshot(self, snapshot):
+        """Reverts a share (in place) to a snapshot."""
+        self.manager.revert_to_snapshot(self, snapshot)
+
 
 class ShareManager(base.ManagerWithFind):
     """Manage :class:`Share` resources."""
@@ -117,11 +123,11 @@ class ShareManager(base.ManagerWithFind):
     def create(self, share_proto, size, snapshot_id=None, name=None,
                description=None, metadata=None, share_network=None,
                share_type=None, is_public=False, availability_zone=None,
-               consistency_group_id=None):
+               share_group_id=None):
         """Create a share.
 
-        :param share_proto: text - share protocol for new share
-            available values are NFS, CIFS, CephFS, GlusterFS and HDFS.
+        :param share_proto: text - share protocol for new share available
+            values are NFS, CIFS, CephFS, GlusterFS, HDFS and MAPRFS.
         :param size: int - size in GiB
         :param snapshot_id: text - ID of the snapshot
         :param name: text - name of new share
@@ -130,8 +136,8 @@ class ShareManager(base.ManagerWithFind):
         :param share_network: either instance of ShareNetwork or text with ID
         :param share_type: either instance of ShareType or text with ID
         :param is_public: bool, whether to set share as public or not.
-        :param consistency_group_id: text - ID of the consistency group to
-            which the share should belong
+        :param share_group_id: text - ID of the share group to which the share
+            should belong
         :rtype: :class:`Share`
         """
         share_metadata = metadata if metadata is not None else dict()
@@ -146,21 +152,24 @@ class ShareManager(base.ManagerWithFind):
             'share_type': common_base.getid(share_type),
             'is_public': is_public,
             'availability_zone': availability_zone,
-            'consistency_group_id': consistency_group_id,
         }
+        if share_group_id:
+            body['share_group_id'] = share_group_id
+
         return self._create('/shares', {'share': body}, 'share')
 
-    @api_versions.wraps("2.22")
+    @api_versions.wraps("2.29")
     @api_versions.experimental_api
     def migration_start(self, share, host, force_host_assisted_migration,
-                        preserve_metadata=True, writable=True,
-                        nondisruptive=False, new_share_network_id=None,
+                        preserve_metadata, writable, nondisruptive,
+                        preserve_snapshots, new_share_network_id=None,
                         new_share_type_id=None):
         return self._action(
             "migration_start", share, {
                 "host": host,
                 "force_host_assisted_migration": force_host_assisted_migration,
                 "preserve_metadata": preserve_metadata,
+                "preserve_snapshots": preserve_snapshots,
                 "writable": writable,
                 "nondisruptive": nondisruptive,
                 "new_share_network_id": new_share_network_id,
@@ -276,6 +285,19 @@ class ShareManager(base.ManagerWithFind):
         """
         return self._action("unmanage", share)
 
+    @api_versions.wraps("2.27")
+    def revert_to_snapshot(self, share, snapshot):
+        """Reverts a share (in place) to a snapshot.
+
+        The snapshot must be the most recent one known to manila.
+        :param share: either share object or text with its ID.
+        :param snapshot: either snapshot object or text with its ID.
+        """
+
+        snapshot_id = common_base.getid(snapshot)
+        info = {'snapshot_id': snapshot_id}
+        return self._action('revert', share, info=info)
+
     def get(self, share):
         """Get a share.
 
@@ -298,8 +320,24 @@ class ShareManager(base.ManagerWithFind):
         share_id = common_base.getid(share)
         return self._update("/shares/%s" % share_id, body)
 
+    @api_versions.wraps("1.0", "2.34")
     def list(self, detailed=True, search_opts=None,
              sort_key=None, sort_dir=None):
+        """Get a list of all shares."""
+        search_opts = search_opts or {}
+        search_opts.pop("export_location", None)
+        return self.do_list(detailed=detailed, search_opts=search_opts,
+                            sort_key=sort_key, sort_dir=sort_dir)
+
+    @api_versions.wraps("2.35")   # noqa
+    def list(self, detailed=True, search_opts=None,
+             sort_key=None, sort_dir=None):
+        """Get a list of all shares."""
+        return self.do_list(detailed=detailed, search_opts=search_opts,
+                            sort_key=sort_key, sort_dir=sort_dir)
+
+    def do_list(self, detailed=True, search_opts=None,
+                sort_key=None, sort_dir=None):
         """Get a list of all shares.
 
         :param detailed: Whether to return detailed share info or not.
@@ -353,13 +391,20 @@ class ShareManager(base.ManagerWithFind):
         if 'is_public' not in search_opts:
             search_opts['is_public'] = True
 
+        export_location = search_opts.pop('export_location', None)
+        if export_location:
+            if uuidutils.is_uuid_like(export_location):
+                search_opts['export_location_id'] = export_location
+            else:
+                search_opts['export_location_path'] = export_location
+
+        query_string = ""
         if search_opts:
-            query_string = urlencode(
-                sorted([(k, v) for (k, v) in list(search_opts.items()) if v]))
-            if query_string:
-                query_string = "?%s" % (query_string,)
-        else:
-            query_string = ''
+            search_opts = utils.unicode_key_value_to_string(search_opts)
+            params = sorted(
+                [(k, v) for (k, v) in list(search_opts.items()) if v])
+            if params:
+                query_string = "?%s" % parse.urlencode(params)
 
         if detailed:
             path = "/shares/detail%s" % (query_string,)
@@ -368,16 +413,16 @@ class ShareManager(base.ManagerWithFind):
 
         return self._list(path, 'shares')
 
-    def delete(self, share, consistency_group_id=None):
+    def delete(self, share, share_group_id=None):
         """Delete a share.
 
         :param share: either share object or text with its ID.
-        :param consistency_group_id: text - ID of the consistency group to
-            which the share belongs to.
+        :param share_group_id: text - ID of the share group to which the share
+            belongs
         """
         url = "/shares/%s" % common_base.getid(share)
-        if consistency_group_id:
-            url += "?consistency_group_id=%s" % consistency_group_id
+        if share_group_id:
+            url += "?share_group_id=%s" % share_group_id
         self._delete(url)
 
     def _do_force_delete(self, share, action_name):
@@ -401,41 +446,30 @@ class ShareManager(base.ManagerWithFind):
             exc_str = ('Invalid CN (common name). Must be 1-64 chars long.')
             raise exceptions.CommandError(exc_str)
 
+    '''
+    for the reference specification for AD usernames, reference below links:
+
+    1:https://docs.microsoft.com/en-us/previous-versions/windows/it-pro/
+            windows-server-2008-R2-and-2008/cc733146(v=ws.11)
+    2:https://docs.microsoft.com/en-us/previous-versions/windows/it-pro/
+            windows-server-2000/bb726984(v=technet.10)
+    '''
     @staticmethod
     def _validate_username(access):
-        valid_username_re = '[\w\.\-_\`;\'\{\}\[\]\\\\]{4,32}$'
+        sole_periods_spaces_re = '[\s|\.]+$'
+        valid_username_re = '.[^\"\/\\\[\]\:\;\|\=\,\+\*\?\<\>]{3,254}$'
         username = access
-        if not re.match(valid_username_re, username):
-            exc_str = ('Invalid user or group name. Must be 4-32 characters '
-                       'and consist of alphanumeric characters and '
-                       'special characters ]{.-_\'`;}[\\')
+
+        if re.match(sole_periods_spaces_re, username):
+            exc_str = ('Invalid user or group name,cannot consist solely '
+                       'of periods or spaces.')
             raise exceptions.CommandError(exc_str)
 
-    @staticmethod
-    def _validate_ip_range(ip_range):
-        ip_range = ip_range.split('/')
-        exc_str = ('Supported ip format examples:\n'
-                   '\t10.0.0.2, 10.0.0.0/24')
-        if len(ip_range) > 2:
+        if not re.match(valid_username_re, username):
+            exc_str = ('Invalid user or group name. Must be 4-255 characters '
+                       'and consist of alphanumeric characters and '
+                       'exclude special characters "/\[]:;|=,+*?<>')
             raise exceptions.CommandError(exc_str)
-        if len(ip_range) == 2:
-            try:
-                prefix = int(ip_range[1])
-                if prefix < 0 or prefix > 32:
-                    raise ValueError()
-            except ValueError:
-                msg = 'IP prefix should be in range from 0 to 32.'
-                raise exceptions.CommandError(msg)
-        ip_range = ip_range[0].split('.')
-        if len(ip_range) != 4:
-            raise exceptions.CommandError(exc_str)
-        for item in ip_range:
-            try:
-                if 0 <= int(item) <= 255:
-                    continue
-                raise ValueError()
-            except ValueError:
-                raise exceptions.CommandError(exc_str)
 
     @staticmethod
     def _validate_cephx_id(cephx_id):
@@ -457,13 +491,20 @@ class ShareManager(base.ManagerWithFind):
             raise exceptions.CommandError(
                 'Ceph IDs may not contain periods.')
 
-    def _validate_access(self, access_type, access, valid_access_types=None):
+    def _validate_access(self, access_type, access, valid_access_types=None,
+                         enable_ipv6=False):
         if not valid_access_types:
             valid_access_types = ('ip', 'user', 'cert')
 
         if access_type in valid_access_types:
             if access_type == 'ip':
-                self._validate_ip_range(access)
+                try:
+                    if enable_ipv6:
+                        ipaddress.ip_network(six.text_type(access))
+                    else:
+                        ipaddress.IPv4Network(six.text_type(access))
+                except ValueError as error:
+                    raise exceptions.CommandError(six.text_type(error))
             elif access_type == 'user':
                 self._validate_username(access)
             elif access_type == 'cert':
@@ -480,13 +521,15 @@ class ShareManager(base.ManagerWithFind):
                    ', '.join(valid_access_types))
             raise exceptions.CommandError(msg)
 
-    def _do_allow(self, share, access_type, access, access_level, action_name):
+    def _do_allow(self, share, access_type, access, access_level, action_name,
+                  metadata=None):
         """Allow access to a share.
 
         :param share: either share object or text with its ID.
         :param access_type: string that represents access type ('ip','domain')
         :param access: string that represents access ('127.0.0.1')
         :param access_level: string that represents access level ('rw', 'ro')
+        :param metadata: A dict of key/value pairs to be set
         """
         access_params = {
             'access_type': access_type,
@@ -494,29 +537,47 @@ class ShareManager(base.ManagerWithFind):
         }
         if access_level:
             access_params['access_level'] = access_level
+        if metadata:
+            access_params['metadata'] = metadata
         access = self._action(action_name, share,
                               access_params)[1]["access"]
         return access
 
     @api_versions.wraps("1.0", "2.6")
-    def allow(self, share, access_type, access, access_level):
+    def allow(self, share, access_type, access, access_level, metadata=None):
         self._validate_access(access_type, access)
         return self._do_allow(
             share, access_type, access, access_level, "os-allow_access")
 
     @api_versions.wraps("2.7", "2.12")  # noqa
-    def allow(self, share, access_type, access, access_level):
+    def allow(self, share, access_type, access, access_level, metadata=None):
         self._validate_access(access_type, access)
         return self._do_allow(
             share, access_type, access, access_level, "allow_access")
 
-    @api_versions.wraps("2.13")  # noqa
-    def allow(self, share, access_type, access, access_level):
+    @api_versions.wraps("2.13", "2.37")  # noqa
+    def allow(self, share, access_type, access, access_level, metadata=None):
         valid_access_types = ('ip', 'user', 'cert', 'cephx')
         self._validate_access(access_type, access, valid_access_types)
         return self._do_allow(
             share, access_type, access, access_level, "allow_access")
 
+    @api_versions.wraps("2.38", "2.44")  # noqa
+    def allow(self, share, access_type, access, access_level, metadata=None):
+        valid_access_types = ('ip', 'user', 'cert', 'cephx')
+        self._validate_access(access_type, access, valid_access_types,
+                              enable_ipv6=True)
+        return self._do_allow(
+            share, access_type, access, access_level, "allow_access")
+
+    @api_versions.wraps("2.45")  # noqa
+    def allow(self, share, access_type, access, access_level, metadata=None):
+        valid_access_types = ('ip', 'user', 'cert', 'cephx')
+        self._validate_access(access_type, access, valid_access_types,
+                              enable_ipv6=True)
+        return self._do_allow(
+            share, access_type, access, access_level, "allow_access", metadata)
+
     def _do_deny(self, share, access_id, action_name):
         """Deny access to a share.
 
@@ -549,7 +610,7 @@ class ShareManager(base.ManagerWithFind):
     def access_list(self, share):
         return self._do_access_list(share, "os-access_list")
 
-    @api_versions.wraps("2.7")  # noqa
+    @api_versions.wraps("2.7", "2.44")  # noqa
     def access_list(self, share):
         return self._do_access_list(share, "access_list")
 
